//===========================================================================
//
// UtcClock
//
// Ted Drain.  tnt1of2-samurize@yahoo.com
//
// FYI: Much of the documentation is from the Samurize help file.  The
// original source is at www.samurize.com.
//
//===========================================================================
#include "UtcClock.h"
#include <cstring>
#include <cstdio>
#include <ctime>

//===========================================================================
//
// STATICS
//
//===========================================================================
   
//===========================================================================
//
// SAMURIZE API FUNCTIONS
//
//===========================================================================

//: Library initialization.
//
// The standard DLLMain function. You can use this function to do
// one-off initialization and de-initialization for your plugin using
// the DLL_PROCESS_ATTACH and DLL_PROCESS_DETACH reasons for call.
//
//= INPUT VARIABLES
//
//- hModule             Handle to the DLL module.  Needed by GUI dialogs
//                      to load resources from.
//- ul_reason_for_call  Indicates why the DLL entry-point function is
//                      being called.  Value will be DLL_PROCESS_ATTACH if
//                      the DLL is being loaded by Samurize (ie. when the
//                      first meter that uses the plugin is created) and
//                      DLL_PROCESS_DETACH if DLL is being unloaded by
//                      Samurize (ie. after the last meter that uses the
//                      plugin has been destroyed)
//- lpReserved          If fdwReason is DLL_PROCESS_ATTACH, lpvReserved is
//                      NULL for dynamic loads and non-NULL for static loads.
//                      If fdwReason is DLL_PROCESS_DETACH, lpvReserved is
//                      NULL if DllMain has been called by using FreeLibrary
//                      and non-NULL if DllMain has been called during process
//                      termination.
//= RETURN VALUE
//
//- This function should return TRUE.
//
BOOL APIENTRY
DllMain( HINSTANCE hModule,
         DWORD ul_reason_for_call,
         LPVOID lpReserved )
{
   switch ( ul_reason_for_call )
   {
   // This is when the plugin is loaded
   case DLL_PROCESS_ATTACH:
      break;

   // This is when the plugin is shut down
   case DLL_PROCESS_DETACH:
      break;
   }

   return TRUE;
}

//: Get the list of functions in the module.
//
// The init() function tells Samurize the names of the functions your
// source plugin provides. It should return a list like so:
// "Func1|Func2|...|FuncN"
//
//= RETURN VALUE
//
//- A list of all the functions your source plugin provides eg. "My
//  Function 1|Some other function|And another"
//
SAMURIZE_PLUGIN_CHAR
init()
{   
   return "GetTime";
}


//: Tell Samurize what type of plugin this is.
//
// If this function is declared, Samurize will assume you are using
// the SDK v2.0.
//
// All source functions your plugin provides will be called with an
// additional first parameter, the meter's unique_id.  dllstartup will
// be called for each meter created that uses your plugin (if you
// provide that function).  dllshutdown when a meter that uses your
// plugin is deleted (if you provide that function)
//
//= RETURN VALUE
//
//- The value you should return is a summation of any or all of the
//  following values:  WM_SAM_VISUALPLUGIN, WM_SAM_INPUTPLUGIN,
//  WM_SAM_SOURCEPLUGIN.
//
SAMURIZE_PLUGIN_INT
dlltype()
{
   // These can be added together for more than one type.
   // WM_SAM_VISUALPLUGIN
   // WM_SAM_INPUTPLUGIN
   return WM_SAM_SOURCEPLUGIN;
}


//: Get general plugin information.
//
// This function should return information about your plugin depending
// on the passed integer value. This information is displayed when the
// 'About' button is pressed in the config editor when your plugin is
// selected.
//
// This function is optional. The input value is an enumeration for
// requesting the following values:
//
// 1: Plugin name 
// 2: Author name 
// 3: Plugin version 
// 4: Creation date 
// 5: Last modified date 
// 6: Website 
// 7: Contact email address 
//
//= RETURN VALUE
//
//- Return the requested information string.
//
SAMURIZE_PLUGIN_CHAR
getinfo( int infoNum )
{
   InfoType infoType = (InfoType)infoNum;
   switch ( infoType )
   {
   case PLUGIN_NAME: return "UtcClock";
   case PLUGIN_AUTHOR: return "Ted Drain";
   case PLUGIN_VERSION: return "1.0";
   case PLUGIN_CREATED: return "18-Feb-2006";
   case PLUGIN_UPDATED: return "18-Feb-2006";
   case PLUGIN_WEBSITE: return "None";
   case PLUGIN_EMAIL: return "tnt1of2-samurize@yahoo.com";
   }

   return "";
}

//: Meter startup function.
//
// dllstartup() is called every time Samurize creates a meter that
// uses this plugin.  Any memory allocation needed on a meter-by-meter
// basis should be done in this function.
//
//- INPUT VARIABLES
//
//- hwnd       The window handle of the instance of Samurize that called
//             this function. 
//- dlltype    The meter 'type' (1 for source plugins, 2 for visual plugins)
//             that is being initialized.
//
//= RETURN VALUE
//
//- This function should return a unique integer ID that Samurize
//  remembers and uses for other functions (this is so multiple
//  instances of Samurize that use this plugin do not conflict). If your
//  plugin is very simple and does not need to keep any global variables
//  that may cause conflicts, just return 0 (or omit this function).
//
SAMURIZE_PLUGIN_INT
dllstartup( HWND hwnd,
            int dlltype )
{
   char* buffer = new char[ 120 ];
   return (int)buffer;
}

//: Meter shutdown function.
//
// dllshutdown() is called every time a meter using the plugin is
// destroyed by Samurize.  Any memory releasing needed on a
// meter-by-meter basis should be done in this function.  If your
// plugin does not need to free any memory, this function may be
// omitted.
//
//= INPUT VARIABLES
//
//- id      The unique ID your plugin assigned to the meter
//          (returned by dllstartup)
//
//= RETURN VALUE
//
//- This function should always return 0.
//
SAMURIZE_PLUGIN_INT
dllshutdown( int id )
{
   if ( id )
   {
      char* buffer = (char*)id;
      delete [] buffer;
   }

   return 0;
}

//: Copy a meter.
//
// pastemeter is called when a meter is pasted in the config
// editor. Your plugin should clone the settings from the source meter
// (indicated by source_id) to the cloned meter (indicated by
// dest_id).
//
//= INPUT VARIABLES
//
//- source_id     The unique ID assigned to the source meter by the
//                dllstartup() function.
//- dest_id       The unique ID assigned to the destination meter by
//                the dllstartup() function
//
//= RETURN VALUE
//
//- This function should return 0.
//
SAMURIZE_PLUGIN_INT
pastemeter( int thesource,
            int thedest )
{
   // Do nothing.  Nothing to copy since we're just using the struct
   // as string buffer.
   return 0;
}

//: Get the required parameters for a module function.
//
// The getparam() function should return a list of the parameters that
// the passed function requires.
//
//= INPUT VARIABLES
//
//- func    The name of the function Samurize needs to know the parameters for.
//
//= RETURN VALUE
//
//- A list of parameters delimited by '|' characters. Note that there
//  needs to be a final '|' after the last parameter. Appending '=' and
//  then a value to the end of a parameter indicates the default value
//  for that parameter.  eg. "Param1|Param2|Param3=SomeDefaultValue|"
//
SAMURIZE_PLUGIN_CHAR
getparam( char* func )
{
   if ( strcmp( func, "GetTime" ) == 0 )
   {
      return "Time Format String|";
   }

   return "|";
}

//: Global plugin configuration.
//
// configure() is called when the user clicks on the "Configure
// Plugin..." button in Samurize.  You can display a popup dialog with
// global settings for your plugin (eg. a box prompting for your
// location for a weather plugin). These settings will be applied for
// all meters that use this source plugin.
//
// Note that it is your responsibility to save these settings to the
// registry/settings file/whatever - they aren't stored by Samurize
// anywhere.
//
//= RETURN VALUE
//
//- This function should always return 0.
//
SAMURIZE_PLUGIN_INT
configure()
{
   return 0;
}

//: Individual meter configuration.
//
// source_configure() is called when the user clicks on the
// "Configure..." button in Samurize.  You can display some kind of
// popup window that allows users to specify various display settings
// for this source plugin. Note that you need to store the settings
// for each meter individually for saving later on.
//
//= INPUT VARIABLES
//
//- id         The unique ID of the meter being configured (that your
//             plugin assigned to the meter as the return value of dllstartup) 
//- func       The name of the function this meter calls from the plugin .
//
//
//= RETURN VALUE
//
//- This function should always return 0.
//
SAMURIZE_PLUGIN_INT
source_configure( int id,
                  char* func )
{
   return 0;
}

//===========================================================================
//
// LIBRARY COMMAND FUNCTIONS
//
//===========================================================================

//: Get the temperature from a digital meter.
//
//= INPUT VARIABLES
//
//- id        The Samurize plugin ID.  This is ID from dllstartup()
//- timeFmt   The time formatting string to use.
//
SAMURIZE_PLUGIN_CHAR
GetTime( int id,
         char* timeFmt )
{
   char* buffer = (char*)id;

   // Read the current clock time.
   time_t currentTime;
   time( &currentTime );

   // Convert to UTC
   struct tm* utcTime = gmtime( &currentTime );

   // Format the time.
   if ( ! timeFmt || strlen( timeFmt ) == 0 )
   {
      strftime( buffer, 120, "%X", utcTime );
   }
   else
   {
      strftime( buffer, 120, timeFmt, utcTime );
   }

   return buffer;
}

